@@ -1684,19 +1684,18 @@ do_create_process_fork(p_options *info, create_method method)
     if ( info->detached )
     { setsid();
 #ifdef HAVE_SETPGID
-    setpgid(pid, pid);
+      setpgid(pid, pid);
 #elif defined(HAVE_SETPGRP)
 #ifdef SETPGRP_VOID
-    setpgrp();
+      setpgrp();
 #else
-    setpgrp(pid, pid);
+      setpgrp(pid, pid);
 #endif
 #endif
-    }
-    else
+    } else
     {
 #ifdef HAVE_PRCTL
-    prctl(PR_SET_PDEATHSIG, SIGTERM);
+      prctl(PR_SET_PDEATHSIG, SIGTERM);
 #endif
     }
 
@@ -1720,7 +1719,11 @@ do_create_process_fork(p_options *info, create_method method)
 	  dup2(fd, 0);
         break;
       case std_std:
+      { int fd = Sfileno(Suser_input);
+        if ( fd > 0 )
+	  dup2(fd, 0);
 	break;
+      }
     }
 					/* stdout */
     switch( info->streams[1].type )
@@ -1735,7 +1738,11 @@ do_create_process_fork(p_options *info, create_method method)
 	  dup2(fd, 1);
         break;
       case std_std:
+      { int fd = Sfileno(Suser_output);
+        if ( fd >= 0 && fd != 1 )
+	  dup2(fd, 1);
 	break;
+      }
     }
 					/* stderr */
     switch( info->streams[2].type )
@@ -1750,7 +1757,11 @@ do_create_process_fork(p_options *info, create_method method)
 	  dup2(fd, 2);
         break;
       case std_std:
+      { int fd = Sfileno(Suser_error);
+        if ( fd >= 0 && fd != 2 )
+	  dup2(fd, 2);
 	break;
+      }
     }
 
     if ( info->envp )
