@@ -3,7 +3,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2017, VU University Amsterdam
+    Copyright (c)  2018, VU University Amsterdam
+			 CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -37,6 +38,7 @@
             rdf_current_prefix/2,       % :Alias, ?URI
             rdf_register_prefix/2,      % +Alias, +URI
             rdf_register_prefix/3,      % +Alias, +URI, +Options
+            rdf_unregister_prefix/1,    % +Alias
             register_file_prefixes/1,   % +Pairs
 
             rdf_current_ns/2,           % :Alias, ?URI
@@ -178,7 +180,7 @@ rdf_db:ns(xsd,     'http://www.w3.org/2001/XMLSchema#').
 %   Register Prefix as an abbreviation for URI. Options:
 %
 %           * force(Boolean)
-%           If =true=, Replace existing namespace alias. Please note
+%           If =true=, replace existing namespace alias. Please note
 %           that replacing a namespace is dangerous as namespaces
 %           affect preprocessing. Make sure all code that depends on
 %           a namespace is compiled after changing the registration.
@@ -254,9 +256,18 @@ order_prefixes(Pairs, Sorted) :-
 prefix_uri_length(_-URI, Len) :-
     atom_length(URI, Len).
 
+%!  rdf_unregister_prefix(+Alias) is det.
+%
+%   Delete a prefix global registration.
+
+rdf_unregister_prefix(Alias) :-
+    must_be(atom, Alias),
+    retractall(rdf_db:ns(Alias, _)).
+
+
 %!  rdf_current_ns(:Prefix, ?URI) is nondet.
 %
-%   @deprecated.  Use rdf_current_prefix/2.
+%   @deprecated  Use rdf_current_prefix/2.
 
 rdf_current_ns(Prefix, URI) :-
     rdf_current_prefix(Prefix, URI).
@@ -266,7 +277,7 @@ rdf_current_ns(Prefix, URI) :-
 %
 %   Register an RDF prefix.
 %
-%   @deprecated. Use rdf_register_prefix/2 or rdf_register_prefix/3.
+%   @deprecated  Use rdf_register_prefix/2 or rdf_register_prefix/3.
 
 rdf_register_ns(Prefix, URI) :-
     rdf_register_prefix(Prefix, URI).
