@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2013-2016, University of Amsterdam
+    Copyright (c)  2013-2018, University of Amsterdam
                               VU University Amsterdam
+                              CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -46,6 +47,8 @@
 :- asserta(user:file_search_path(foreign, '.')).
 :- asserta(user:file_search_path(foreign, '../http')).
 :- asserta(user:file_search_path(foreign, '../sgml')).
+:- prolog_load_context(directory, D),
+   asserta(user:file_search_path(library, D)).
 
 :- use_module(library(plunit)).
 :- use_module(library(ssl)).
@@ -54,7 +57,7 @@
 :- use_module(library(error)).
 :- use_module(library(readutil)).
 :- use_module(library(socket)).
-:- use_module(https).
+:- use_module(library(https)).
 
 %:- debug(connection).
 %:- debug(certificate).
@@ -67,7 +70,7 @@ test_ssl :-
                 ssl_keys,
                 https_open,
                 ssl_certificates,
-                evp_encrypt
+                crypto_data_encrypt
               ]).
 :- dynamic
     option/1,                       % Options to test
@@ -76,6 +79,15 @@ test_ssl :-
 run_network_tests :-
     \+ getenv('USE_PUBLIC_NETWORK_TESTS', false).
 
+%!  cert_file(+File, -Absolute)
+%
+%   Find an absolute path to the certificates in the `etc` directory.
+
+cert_file(File, Abs) :-
+    source_file(test_ssl, MyFile),
+    file_directory_name(MyFile, MyDir),
+    atomic_list_concat([MyDir, File], /, Abs).
+
 :- begin_tests(https_open, [condition(run_network_tests)]).
 
 test(readme, Title == "# SWI-Prolog SSL interface") :-
@@ -135,9 +147,9 @@ test(trip_private_public, In == Out) :-
               ( skip_to_pem_cert(S2),
                 load_certificate(S2, Cert)
               )),
-    memberchk(key(PublicKey), Cert),
-    rsa_private_encrypt(PrivateKey, In, Encrypted),
-    rsa_public_decrypt(PublicKey, Encrypted, Out).
+    certificate_field(Cert, public_key(PublicKey)),
+    rsa_private_encrypt(PrivateKey, In, Encrypted, []),
+    rsa_public_decrypt(PublicKey, Encrypted, Out, []).
 test(trip_private_public, In == Out) :-
     numlist(1040, 1060, L),
     string_codes(In, L),
@@ -147,9 +159,9 @@ test(trip_private_public, In == Out) :-
               ( skip_to_pem_cert(S2),
                 load_certificate(S2, Cert)
               )),
-    memberchk(key(PublicKey), Cert),
-    rsa_private_encrypt(PrivateKey, In, Encrypted),
-    rsa_public_decrypt(PublicKey, Encrypted, Out).
+    certificate_field(Cert, public_key(PublicKey)),
+    rsa_private_encrypt(PrivateKey, In, Encrypted, []),
+    rsa_public_decrypt(PublicKey, Encrypted, Out, []).
 test(trip_public_private, In == Out) :-
     In = "Hello World!",
     from_file('tests/test_certs/server-key.pem', S1,
@@ -158,9 +170,9 @@ test(trip_public_private, In == Out) :-
               ( skip_to_pem_cert(S2),
                 load_certificate(S2, Cert)
               )),
-    memberchk(key(PublicKey), Cert),
-    rsa_public_encrypt(PublicKey, In, Encrypted),
-    rsa_private_decrypt(PrivateKey, Encrypted, Out).
+    certificate_field(Cert, public_key(PublicKey)),
+    rsa_public_encrypt(PublicKey, In, Encrypted, []),
+    rsa_private_decrypt(PrivateKey, Encrypted, Out, []).
 
 :- end_tests(ssl_keys).
 
@@ -179,17 +191,23 @@ options_errmsg(Options, Msg) :-
 sni(SSL, _, SSL).
 
 test(cert_mismatch, Msg == 'key values mismatch') :-
-    options_errmsg([certificate_file('etc/server/server-cert.pem'),
-                    key_file('etc/client/client-key.pem'),
+    cert_file('etc/server/server-cert.pem', CertFile),
+    cert_file('etc/client/client-key.pem', KeyFile),
+    options_errmsg([certificate_file(CertFile),
+                    key_file(KeyFile),
                     password(apenoot2)], Msg).
 test(cert_mismatch, Msg == 'key values mismatch') :-
-    read_file_to_string('etc/server/server-key.pem', Key, []),
-    read_file_to_string('etc/client/client-cert.pem', Cert, []),
+    cert_file('etc/server/server-key.pem', KeyFile),
+    cert_file('etc/client/client-cert.pem', CertFile),
+    read_file_to_string(KeyFile, Key, []),
+    read_file_to_string(CertFile, Cert, []),
     options_errmsg([certificate_key_pairs([Cert-Key]),
                     password('apenoot1')], Msg).
 test(cert_mismatch, Msg == 'key values mismatch') :-
-    read_file_to_string('etc/server/server-key.pem', Key, []),
-    read_file_to_string('etc/client/client-cert.pem', Cert, []),
+    cert_file('etc/server/server-key.pem', KeyFile),
+    cert_file('etc/client/client-cert.pem', CertFile),
+    read_file_to_string(KeyFile, Key, []),
+    read_file_to_string(CertFile, Cert, []),
     options_errmsg([certificate_key_pairs([Cert-Key]),
                     password('apenoot1'),
                     sni_hook(sni)], Msg).
@@ -197,19 +215,23 @@ test(cert_mismatch, Msg == 'key values mismatch') :-
 % missing certificate (key specified, with and without sni)
 
 test(missing_cert, Msg == 'no certificate assigned') :-
-    options_errmsg([key_file('etc/server/server-key.pem'),
+    cert_file('etc/server/server-key.pem', KeyFile),
+    options_errmsg([key_file(KeyFile),
                     password(apenoot1)], Msg).
 test(missing_cert, Msg == 'no certificate assigned') :-
-    options_errmsg([key_file('etc/server/server-key.pem'),
+    cert_file('etc/server/server-key.pem', KeyFile),
+    options_errmsg([key_file(KeyFile),
                     password(apenoot1),
                     sni_hook(sni)], Msg).
 
 % missing key (certificate specified, with and without sni)
 
 test(missing_key, Msg == 'no private key assigned') :-
-    options_errmsg([certificate_file('etc/server/server-cert.pem')], Msg).
+    cert_file('etc/server/server-cert.pem', CertFile),
+    options_errmsg([certificate_file(CertFile)], Msg).
 test(missing_key, Msg == 'no private key assigned') :-
-    options_errmsg([certificate_file('etc/server/server-cert.pem'),
+    cert_file('etc/server/server-cert.pem', CertFile),
+    options_errmsg([certificate_file(CertFile),
                     sni_hook(sni)], Msg).
 
 :- end_tests(ssl_options).
@@ -254,7 +276,8 @@ ssl_server :-
                  *******************************/
 
 :- dynamic
-    stop_server/0.
+    stop_server/0,
+    server_port/1.
 
 make_server(SSL, Socket) :-
     ssl_context(server, SSL,
@@ -266,10 +289,12 @@ make_server(SSL, Socket) :-
 %                     password('apenoot1'),
                   pem_password_hook(get_server_pwd)
                 ]),
-    Port = 1111,
+%   Port = 1111,
     tcp_socket(Socket),
     tcp_setopt(Socket, reuseaddr),
     tcp_bind(Socket, localhost:Port),
+    retractall(server_port(_)),
+    asserta(server_port(Port)),
     tcp_listen(Socket, 5).
 
 server_loop(SSL, Server) :-
@@ -342,7 +367,7 @@ client :-
     client_loop(SSL).
 
 client_loop(SSL) :-
-    Port = 1111,
+    server_port(Port),
     tcp_connect(localhost:Port, StreamPair, []),
     stream_pair(StreamPair, Read, Write),
     ssl_negotiate(SSL, Read, Write, In, Out),
@@ -397,22 +422,29 @@ get_client_pwd(_SSL, "apenoot2") :-
 
 :- dynamic
     certificate_verification_result/1,
-    stop_server/1.
+    stop_server/1,
+    test_port/1.
 
+:- meta_predicate
+    do_verification_test(+, 0, -, -).
 
 do_verification_test(Key, Goal, VerificationResults, Status) :-
+%   Port = 2443,
     retractall(stop_server(_)),
     tcp_socket(ServerFd),
     tcp_setopt(ServerFd, reuseaddr),
-    tcp_bind(ServerFd, 2443),
+    tcp_bind(ServerFd, Port),
     tcp_listen(ServerFd, 5),
-    ( setup_call_cleanup(thread_create(verification_server(Key, ServerFd, Id), Id, []),
-                         catch(Goal,
-                               Exception,
-                               Status = error(Exception)),
-                         stop_verification_server(Id))->
-        ignore(Status = true)
-    ; Status = fail
+    retractall(test_port(_)),
+    asserta(test_port(Port)),
+    (   setup_call_cleanup(
+            thread_create(verification_server(Key, ServerFd, Id), Id, []),
+            catch(Goal,
+                  Exception,
+                  Status = error(Exception)),
+            stop_verification_server(Id))
+    ->  ignore(Status = true)
+    ;   Status = fail
     ),
     findall(VerificationResult,
             retract(certificate_verification_result(VerificationResult)),
@@ -421,7 +453,8 @@ do_verification_test(Key, Goal, VerificationResults, Status) :-
 stop_verification_server(Id):-
     assert(stop_server(Id)),
     tcp_socket(S),
-    catch((tcp_connect(S, localhost:2443, Read, Write),
+    test_port(Port),
+    catch((tcp_connect(S, localhost:Port, Read, Write),
            close(Write, [force(true)]),
            close(Read, [force(true)])),
           _,
@@ -471,15 +504,22 @@ dispatch_client(SSL, PlainIn, PlainOut):-
     flush_output(SSLOut),
     call_cleanup(close(SSLOut), close(SSLIn)).
 
-try_ssl_client(Hostname, Port, Hook):-
-    try_ssl_client(Hostname, Port, Hook, []).
+:- meta_predicate
+    try_ssl_client(+, 5),
+    try_ssl_client(+, 5, +).
 
-try_ssl_client(Hostname, Port, Hook, Options):-
+try_ssl_client(Hostname, Hook):-
+    try_ssl_client(Hostname, Hook, []).
+
+try_ssl_client(Hostname, Hook, Options):-
+    test_port(Port),
     ssl_context(client, SSL,
                 [ host(Hostname),
                   port(Port),
                   cert_verify_hook(Hook),
-                  cacert_file('tests/test_certs/rootCA/cacert.pem')|Options]),
+                  cacert_file('tests/test_certs/rootCA/cacert.pem')
+                | Options
+                ]),
     % Always connect to localhost
     verify_client(localhost:Port, SSL).
 
@@ -532,40 +572,40 @@ test_crl_hook(_SSL, Cert, _Chain, _Tail, revoked):-
     setup_call_cleanup(open('tests/test_certs/rootCA-crl.pem', read, Stream),
                        load_crl(Stream, CRL),
                        close(Stream)),
-    memberchk(serial(Serial), Cert),
+    certificate_field(Cert, serial(Serial)),
     memberchk(revocations(Revocations), CRL),
     \+memberchk(revoked(Serial, _RevocationTime), Revocations).
 
 
 test('Valid certificate, correct hostname in CN, signed by trusted CA', VerificationResults:Status == [verified, verified]:true):-
-    do_verification_test(1, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(1, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Valid certificate, correct hostname in SAN, signed by trusted CA', VerificationResults:Status == [verified, verified]:true):-
-    do_verification_test(2, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(2, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Valid certificate, incorrect hostname in CN, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified, verified]:true):-
-    do_verification_test(3, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(3, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Valid certificate, incorrect hostname in SAN and CN, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified, verified]:true):-
-    do_verification_test(4, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(4, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Valid certificate, correct wildcard hostname in SAN, signed by trusted CA', VerificationResults:Status == [verified, verified]:true):-
-    do_verification_test(5, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(5, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Valid certificate, incorrect wildcard hostname in SAN, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified, verified]:true):-
-    do_verification_test(6, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(6, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Valid certificate, wildcard hostname in SAN with wildcard too high, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified, verified]:true):-
-    do_verification_test(7, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(7, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Valid certificate, wildcard hostname in SAN with wildcard too low, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified, verified]:true):-
-    do_verification_test(8, try_ssl_client('www.bad.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(8, try_ssl_client('www.bad.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Valid certificate, wildcard hostname in SAN with wildcard in right level of domain, signed by trusted CA', VerificationResults:Status == [verified, verified]:true):-
-    do_verification_test(9, try_ssl_client('www.good.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(9, try_ssl_client('www.good.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Valid certificate, illegal wildcard hostname in CN, signed by trusted CA', VerificationResults:Status == [hostname_mismatch, verified, verified]:true):-
-    do_verification_test(10, try_ssl_client('www.good.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(10, try_ssl_client('www.good.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Hostname containing embedded NULL, signed by trusted CA',
      [ true(VerificationResults:Status ==
@@ -573,16 +613,16 @@ test('Hostname containing embedded NULL, signed by trusted CA',
        condition((size_file('tests/test_certs/11-cert.pem', Size),
                   Size > 0))
      ]):-
-    do_verification_test(11, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(11, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Certificate which has expired, signed by trusted CA', VerificationResults:Status == [verified, expired, verified]:true):-
-    do_verification_test(12, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(12, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Certificate which is not yet valid, signed by trusted CA', VerificationResults:Status == [verified, not_yet_valid, verified]:true):-
-    do_verification_test(13, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(13, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Certificate is not issued by trusted CA'):-
-    do_verification_test(14, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status),
+    do_verification_test(14, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status),
     ( VerificationResults:Status == [unknown_issuer, unknown_issuer]:true ->
         % OpenSSL 1.0.2 and above
         true
@@ -592,60 +632,60 @@ test('Certificate is not issued by trusted CA'):-
     ).
 
 test('Certificate is issued by trusted CA but has been altered so signature is wrong', VerificationResults:Status == [verified, bad_signature, verified]:true):-
-    do_verification_test(15, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(15, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Certificate is not intended for SSL', VerificationResults:Status == [bad_certificate_use, verified, verified]:true):-
-    do_verification_test(17, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(17, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Certificate signed not-explicitly-trusted intermediary requiring us to follow the chain', VerificationResults:Status == [verified, verified, verified]:true):-
-    do_verification_test(18, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(18, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Chain involving expired intermediary', VerificationResults:Status == [verified, expired, verified, verified]:true):-
-    do_verification_test(20, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(20, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Chain involving not-yet-valid intermediary', VerificationResults:Status == [verified, not_yet_valid, verified, verified]:true):-
-    do_verification_test(21, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(21, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Chain involving intermediary not authorized to sign certificates', VerificationResults:Status == [invalid_ca, bad_certificate_use, verified, verified, verified]:true):-
-    do_verification_test(22, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(22, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Confirm that a failure in the verification callback triggers a connection abort', Status = error(_)):-
-    do_verification_test(17, try_ssl_client('www.example.com', 2443, fail_verify_hook), _, Status).
+    do_verification_test(17, try_ssl_client('www.example.com', fail_verify_hook), _, Status).
 
 test('Confirm that an exception in the verification callback triggers a connection abort', Status = error(_)):-
-    do_verification_test(17, try_ssl_client('www.example.com', 2443, abort_verify_hook), _, Status).
+    do_verification_test(17, try_ssl_client('www.example.com', abort_verify_hook), _, Status).
 
 
 test('Certificate has a CRL but has not been revoked. We do not provide the CRL', VerificationResults:Status == [unknown_crl, unknown_crl, verified, verified]:true):-
-    do_verification_test(23, try_ssl_client('www.example.com', 2443, test_verify_hook, [require_crl(true)]), VerificationResults, Status).
+    do_verification_test(23, try_ssl_client('www.example.com', test_verify_hook, [require_crl(true)]), VerificationResults, Status).
 
 test('Certificate has a CRL but has not been revoked. We do provide the CRL', VerificationResults:Status == [verified, verified]:true):-
-    do_verification_test(23, try_ssl_client('www.example.com', 2443, test_verify_hook, [require_crl(true), crl(['tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
+    do_verification_test(23, try_ssl_client('www.example.com', test_verify_hook, [require_crl(true), crl(['tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
 
 test('Certificate has a CRL and has been revoked. We do not provide the CRL', VerificationResults:Status == [unknown_crl, unknown_crl, verified, verified]:true):-
-    do_verification_test(24, try_ssl_client('www.example.com', 2443, test_verify_hook, [require_crl(true)]), VerificationResults, Status).
+    do_verification_test(24, try_ssl_client('www.example.com', test_verify_hook, [require_crl(true)]), VerificationResults, Status).
 
 test('Certificate has a CRL and has been revoked. We do provide the CRL', VerificationResults:Status == [revoked, verified, verified]:true):-
-    do_verification_test(24, try_ssl_client('www.example.com', 2443, test_verify_hook, [require_crl(true), crl(['tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
+    do_verification_test(24, try_ssl_client('www.example.com', test_verify_hook, [require_crl(true), crl(['tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
 
 test('Certificate has a CRL but we want to ignore it', VerificationResults:Status == [verified, verified]:true):-
-    do_verification_test(24, try_ssl_client('www.example.com', 2443, test_verify_hook), VerificationResults, Status).
+    do_verification_test(24, try_ssl_client('www.example.com', test_verify_hook), VerificationResults, Status).
 
 test('Certificate has an illegal CRL', VerificationResults:Status == [bad_certificate_use, verified, verified, verified]:true):-
-    do_verification_test(25, try_ssl_client('www.example.com', 2443, test_verify_hook, [require_crl(true), crl(['tests/test_certs/25-crl.pem', 'tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
+    do_verification_test(25, try_ssl_client('www.example.com', test_verify_hook, [require_crl(true), crl(['tests/test_certs/25-crl.pem', 'tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
 
 test('Intermediate CA has revoked the certificate', VerificationResults:Status == [revoked, verified, verified, verified]:true):-
-    do_verification_test(26, try_ssl_client('www.example.com', 2443, test_verify_hook, [require_crl(true), crl(['tests/test_certs/26-crl.pem', 'tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
+    do_verification_test(26, try_ssl_client('www.example.com', test_verify_hook, [require_crl(true), crl(['tests/test_certs/26-crl.pem', 'tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
 
 test('root CA has revoked the intermediate CA', VerificationResults:Status == [revoked, verified, verified, verified]:true):-
-    do_verification_test(27, try_ssl_client('www.example.com', 2443, test_verify_hook, [require_crl(true), crl(['tests/test_certs/27-crl.pem', 'tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
+    do_verification_test(27, try_ssl_client('www.example.com', test_verify_hook, [require_crl(true), crl(['tests/test_certs/27-crl.pem', 'tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
 
 
 test('Accept a non-revoked certificate ourselves in a callback', VerificationResults:Status == []:true):-
-    do_verification_test(23, try_ssl_client('www.example.com', 2443, test_crl_hook, [require_crl(true), crl(['tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
+    do_verification_test(23, try_ssl_client('www.example.com', test_crl_hook, [require_crl(true), crl(['tests/test_certs/rootCA-crl.pem'])]), VerificationResults, Status).
 
 test('Reject a revoked certificate ourselves in a callback', Status = error(_)):-
-    do_verification_test(24, try_ssl_client('www.example.com', 2443, test_crl_hook, [require_crl(true), crl(['tests/test_certs/rootCA-crl.pem'])]), _, Status).
+    do_verification_test(24, try_ssl_client('www.example.com', test_crl_hook, [require_crl(true), crl(['tests/test_certs/rootCA-crl.pem'])]), _, Status).
 
 % It would be really nice if there were some way of adding the CRL to the context and retrying, but I dont think this is possible.
 % Looking at the code in x509_vfy.c, once the callback is called for X509_V_ERR_UNABLE_TO_GET_CRL, regardless of the callback status, it goes to err, skipping the
@@ -656,17 +696,17 @@ test('Reject a revoked certificate ourselves in a callback', Status = error(_)):
 :- end_tests(ssl_certificates).
 
 
-:- begin_tests(evp_encrypt).
+:- begin_tests(crypto_data_encrypt).
 
 test(roundtrip, RecoveredText == Text) :-
     Key  = "sixteenbyteofkey",
     IV   = "sixteenbytesofiv",
     Text = "this is some input",
 
-    evp_encrypt(Text, 'aes-128-cbc', Key, IV, CipherText, []),
-    evp_decrypt(CipherText, 'aes-128-cbc', Key, IV, RecoveredText, []).
+    crypto_data_encrypt(Text, 'aes-128-cbc', Key, IV, CipherText, []),
+    crypto_data_decrypt(CipherText, 'aes-128-cbc', Key, IV, RecoveredText, []).
 
-:- end_tests(evp_encrypt).
+:- end_tests(crypto_data_encrypt).
 
 
                  /*******************************
@@ -674,15 +714,15 @@ test(roundtrip, RecoveredText == Text) :-
                  *******************************/
 
 is_certificate(Cert) :-
-    is_list(Cert),
-    memberchk(version(V), Cert), integer(V),
-    memberchk(notbefore(NB), Cert), integer(NB),
-    memberchk(notafter(NA), Cert), integer(NA),
-    memberchk(subject(Subj), Cert), is_subject(Subj),
-    memberchk(hash(H), Cert), is_hex_string(H),
-    memberchk(signature(S), Cert), is_hex_string(S),
-    memberchk(issuer_name(Issuer), Cert), is_issuer(Issuer),
-    memberchk(key(K), Cert), is_public_key(K).
+    blob(Cert, ssl_certificate),
+    certificate_field(Cert, version(V)), integer(V),
+    certificate_field(Cert, not_before(NB)), integer(NB),
+    certificate_field(Cert, not_after(NA)), integer(NA),
+    certificate_field(Cert, subject(Subj)), is_subject(Subj),
+    certificate_field(Cert, hash(H)), is_hex_string(H),
+    certificate_field(Cert, signature(S)), is_hex_string(S),
+    certificate_field(Cert, issuer(Issuer)), is_issuer(Issuer),
+    certificate_field(Cert, public_key(K)), is_public_key(K).
 
 is_subject(Subj) :-
     is_list(Subj),
