@@ -101,6 +101,7 @@
 #define MSG_MUTEX_GC		 56
 #define MSG_REC_ATTVAR		 57
 #define MSG_TTY			 58
+#define MSG_OS_DIR		 59
 
 #define MSG_READ_TOKEN		 60
 
@@ -196,6 +197,7 @@
 #define MSG_TABLING_EXCEPTION	 312
 #define MSG_TABLING_SHARED	 313
 #define MSG_TABLING_ABOLISH	 314
+#define MSG_TABLING_CALL_SUBSUMPTION 315
 
 #define TABLING_NO_EARLY_COMPLETION 320
 #define TABLING_NO_SIMPLIFY	 321		/* these affect behaviour */
