@@ -147,7 +147,12 @@ query_loop(atom_t goal, int loop)
     if ( fid ) PL_discard_foreign_frame(fid);
     if ( !except )
       break;
-  } while(loop);
+  } while(loop && !LD->exit_requested);
+
+  DEBUG(MSG_CLEANUP_THREAD,
+	if ( LD->exit_requested )
+	Sdprintf("Thread %d: leaving REPL loop due to exit_requested\n",
+		 PL_thread_self()));
 
   return rc;
 }
